brands = {
#large corporations
    'apple',
    'google',
    'microsoft',
#social media
    'facebook',
    'twitter',
    'tumblr',
    'reddit',
    'youtube',
    'linkedin',
    'instagram',
    'flickr',
    'whatsapp',
#banking
    'paypal',
    'bankofamerica',
    'moneygram',
    'westernunion',
    'wellsfargo',
    'citigroup',
    'santander',
    'morganstanley',
    'barclays',
    'hsbc',
    'scottrade',
    'ameritrade',
    'merilledge',
    'chase',
#email
    'gmail',
    'hotmal',
    'protonmail',
    'yahoo',
    'outlook',
    'office365',
#e-commerce
    'amazon',
    'overstock',
    'alibaba',
    'aliexpress',
    'leboncoin',
    'ebay',
    'walmart',
#other
    'netflix',
    'skype',
    'github',
    'onedrive',
    'dropbox',
#crytocurrency
    'blockchain',
}

popular_keywords = {
    'login',
    'log-in',
    'sign-in',
    'signin',
    'account',
    'verification',
    'verify',
    'webscr',
    'password',
    'credential',
    'support',
    'activity',
    'security',
    'update',
    'authentication',
    'authenticate',
    'authorize',
    'wallet',
    'alert',
    'purchase',
    'transaction',
    'recover',
    'unlock',
    'confirm',
    'live',
    'office',
    'service',
    'manage',
    'portal',
    'invoice',
    'secure',
    'customer',
    'client',
    'bill',
    'online',
    'safe',
    'form',
    'bank',
    'reactivate'
}

tlds = {
    'ga',
    'gq',
    'ml',
    'cf',
    'tk',
    'xyz',
    'pw',
    'cc',
    'club',
    'work',
    'top',
    'support',
    'bank',
    'info',
    'study',
    'click',
    'country',
    'stream',
    'gdn',
    'mom',
    'xin',
    'kim',
    'men',
    'loan',
    'download',
    'racing',
    'online',
    'center',
    'ren',
    'gb',
    'win',
    'review',
    'vip',
    'party',
    'tech',
    'science',
    'business'
}

